package console

import (
	"bufio"
	"context"
	"doc-service/domain/entity/serversitemicros/docservercallback"
	"doc-service/domain/entity/serversitemicros/docserverproject"
	"doc-service/infra/db"
	"fmt"
	"github.com/nacos-group/nacos-sdk-go/v2/common/logger"
	"gitlab.galaxy-immi.com/Backend-group/go-com/third/log"
	"os"
	"strings"
	"time"
)

func recoverPanicCallBack() {
	if r := recover(); r != nil {
		logger.Error("执行请求回调异常", r)
	}
}

// 执行回调
func DoCallback() {
	defer recoverPanicCallBack()
	fmt.Println("开始请求回调...")
	ctxCall := context.Background()
	tickerCallback := time.NewTicker(20 * time.Second)
	defer tickerCallback.Stop()

	go func() {
		for {
			select {
			case <-tickerCallback.C:
				//执行转换
				i := callbackData(ctxCall)
				if i == 0 {
					fmt.Println("暂无回调数据: " + time.Now().Format("2006-01-02 15:04:05"))
				}
			}
		}
	}()

	// 监听命令行输入
	reader := bufio.NewReader(os.Stdin)
	for {
		text, _ := reader.ReadString('\n')
		text = strings.TrimSpace(text)

		if text == "exit" {
			fmt.Println("Exiting process...")
			os.Exit(0)
		}
	}
}

func callbackData(ctx context.Context) int8 {
	defer recoverPanicCallBack()
	//列表
	callBackData, err := db.GetServerSiteMicrosDB().DocServerProject.Query().Where(
		docserverproject.StatusEQ(2),
	).All(ctx)

	if err != nil {
		return 0
	}

	if len(callBackData) <= 0 {
		return 0
	}

	for _, item := range callBackData {
		//执行转换
		res1, msg := callBackUrl(ctx, item.ID)
		var status int32
		if res1 {
			status = 3
		} else {
			status = 4
		}
		_, errUpdate := db.GetServerSiteMicrosDB().DocServerProject.UpdateOneID(item.ID).SetStatus(status).SetCallbackRes(msg).Save(ctx)
		if errUpdate != nil {
			log.Info("回调请求修改状态出错" + errUpdate.Error())
			continue
		}
		log.Info("请求回调完成: " + string(item.ID))
		fmt.Printf("请求回调完成:   %d\n", item.ID)
	}

	return 1
}

// 执行请求
func callBackUrl(c context.Context, projectId int32) (bool, string) {
	defer recoverPanicCallBack()
	callInfo, callErr := db.GetServerSiteMicrosDB().DocServerCallback.Query().Where(
		docservercallback.ServerProjectID(projectId),
	).First(c)

	var msg string
	var res bool
	res = true
	msg = "success"
	if callErr != nil {
		logger.Error("获取回调单条出错", callErr.Error())
		msg = callErr.Error()
		res = false
	} else {
		if callInfo == nil {
			msg = "回调信息为空"
			res = false
		}
		if callInfo.CallbackURL == "" {
			msg = "回调地址为空"
			res = false
		}
	}

	if res {
		//回调
	}

	return res, msg
}
